#include <glib.h>
#if HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif
#if defined(HAS_PYTHON)
#include <Python.h>
#endif

#include "AlibavaConfig.h"
#include "GtkUtils.h"
#include "PluginHook.h"
#include "utils.h"
#include <iostream>



void CalibrationGUI::user_init()
{
    int2entry("txtStart", _cnf.from());
    int2entry("txtEnd", _cnf.to());
    int2entry("txtNpulse", _cnf.npts());
    int2entry("txtSamples", _cnf.nevts());
    int2entry("txtCalDelay", _cnf.delay());
    int2entry("txtCharge", _cnf.charge());

    if ( _cnf.scan_charge() )
        activate_button("btnScanVar", true );
    else
        activate_button("btnDelayScan", true );


    if (firmware()==0)
    {
        gtk_widget_set_sensitive( get_widget("txtCalDelay"), 0 );
        gtk_widget_set_sensitive( get_widget("btnDelayScan"), 0 );
    }

}

void CalibrationGUI::process_text(const std::string &s, GtkWidget *w)
{
    if (s=="txtSamples")
    {
        _cnf.nevts(entry2int(s) );
    }
    else
    {
        if (s=="txtEnd")
        {
            int max = btn_active("btnScanVar") ? 261375 : 255 ;
            int val = entry2int(s);
            if (val>max)
                val = max;

            _cnf.to( val );
        }
        else if (s=="txtNpulse")
        {
            _cnf.npts(entry2int(s) );
            if (_cnf.npts()>255 )
                _cnf.npts(255);
        }
        else if (s=="txtStart")
        {
            int val = entry2int(s, 0);
            if (val<0)
                val = 0;
            _cnf.from( val );
        }
        else if (s=="txtCalDelay")
        {
            _cnf.delay( entry2int(s) );
        }
        else if (s=="txtCharge")
        {
            _cnf.charge( entry2int(s) );
        }
    }
    check_values();
    _cnf.adjust();
    user_init();
}

void CalibrationGUI::check_values()
{
    _cnf.to( entry2int("txtEnd") );
    _cnf.from( entry2int("txtStart") );
    _cnf.npts(entry2int("txtNpulse") );

    _cnf.scan_charge( btn_active("btnScanVar") );
    _cnf.set_min_step( _cnf.scan_charge() ? 1024 : 1 );

    unsigned int max = _cnf.scan_charge() ? 261375 : 255 ;
    if (_cnf.to() > max)
        _cnf.to( max );

    if ( _cnf.from() < 0 )
        _cnf.from(0);

    if ( _cnf.npts()>255 )
        _cnf.npts(255);

}

void CalibrationGUI::get_values()
{
    check_values();
    _cnf.adjust();
    _cnf.charge( entry2int("txtCharge") );
    _cnf.delay( entry2int("txtCalDelay") );
    _cnf.scan_charge( btn_active("btnScanVar") );
}

void LaserSyncGUI::user_init()
{
    int2spin("spinFrom", _cnf.from());
    int2spin("spinTo", _cnf.to());
    int2spin("spinStep", _cnf.step());
    int2entry("txtSamples", _cnf.nevts());
}

void LaserSyncGUI::set_values()
{
    _cnf.from(spin2int("spinFrom") );
    _cnf.to(spin2int("spinTo") );
    _cnf.set_npts(spin2int("spinStep") );
    _cnf.nevts(entry2int("txtSamples") );
}

void BeetleConfigGUI::user_init()
{
    BeetleConfig::iterator ip;
    for (ip=_cnf.begin(); ip!=_cnf.end(); ++ip)
    {
        int2entry(ip->first, 0xff & ip->second);
    }
    activate_button("btnMaskChip0", _cnf.mask() & 1);
    activate_button("btnMaskChip1", _cnf.mask() & 2);

    if (firmware()==0)
    {
        gtk_widget_set_sensitive( get_widget("btnMaskChip0"), 0 );
        gtk_widget_set_sensitive( get_widget("btnMaskChip1"), 0 );
    }
}

void BeetleConfigGUI::set_values()
{
    BeetleConfig::iterator ip;
    int ival = 0;
    for (ip=_cnf.begin(); ip!=_cnf.end(); ++ip)
    {
        ip->second = _cnf.addr(ival) | (entry2int(ip->first) & 0xff);
        ival ++;
    }
    short mask = 0;
    if ( btn_active("btnMaskChip0") )
        mask |= 1;
    if ( btn_active("btnMaskChip1") )
        mask |= 2;

    _cnf.mask(mask);
}

void LaserConfigGUI::user_init()
{
    int2spin("spinDelay", _cnf.delay());
}

void LaserConfigGUI::set_values()
{
    _cnf.delay(spin2int("spinDelay"));
}

void TriggerConfigGUI::user_init()
{
    const char *spins[] = { "spinDACa", "spinDACb", "spinDACc", "spinDACd", 0 };
    const char *scales[] = { "scaleDACa", "scaleDACb", "scaleDACc", "scaleDACd", 0 };
    int i;
    for (i=0;spins[i]; i++)
    {
        gtk_range_set_adjustment(GTK_RANGE(get_widget(scales[i])),
                                 gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget(spins[i])))
                             );
    }
    click_button(_cnf.trigger_and() ? "rbtnAND" : "rbtnOR");
    click_button(_cnf.pulse() ? "rbtnTriggerPulse" : "rbtnTriggerIn");
    int2spin("spinDACa", _cnf.dacA() );
    int2spin("spinDACb", _cnf.dacB() );
    int2spin("spinDACc", _cnf.dacC() );
    int2spin("spinDACd", _cnf.dacD() );
}

void TriggerConfigGUI::set_values()
{
    _cnf.trigger_and(btn_active("rbtnAND") );
    _cnf.pulse(btn_active("rbtnTriggerPulse") );
    _cnf.dacA(spin2int("spinDACa") );
    _cnf.dacB(spin2int("spinDACb") );
    _cnf.dacC(spin2int("spinDACc") );
    _cnf.dacD(spin2int("spinDACd") );
}

void DAQConfigGUI::user_init()
{
    int2entry("txtMonChannel", _cnf.chan());
    int2entry("txtSampleSize", _cnf.sample_size());
    int2entry("entryMaxEvents", _cnf.nevts());
    int2entry("txtDelay", _cnf.delay());
}

void DAQConfigGUI::set_values()
{
    _cnf.chan(entry2int("txtMonChannel") );
    _cnf.sample_size(entry2int("txtSampleSize") );
    _cnf.nevts( entry2int("entryMaxEvents"));
    _cnf.delay( entry2int("txtDelay"));
}

void DAQConfigGUI::process_text(const std::string &s, GtkWidget *w)
{
    if (s=="entryMaxEvents")
    {
        int uval = 0;
        if ( s.empty() )
        {
            uval = G_MAXINT;
        }
        else
        {
            uval = entry2int(s, G_MAXINT);
        }
        _cnf.nevts(uval);
    }
}


void AnalysisConfigGUI::user_init()
{
    double2entry("txtPedWeight", _cnf.ped_weight());
    double2entry("txtNoiseWeight", _cnf.noise_weight());
    double2entry("txtSeedCut", _cnf.seed_cut());
    double2entry("txtNeighCut", _cnf.neigh_cut());
    double2entry("txtSNCut", _cnf.sn_cut());
    click_button(_cnf.do_cmmd() ? "btnCmmdActive" : "btnCmmdInactive");
}



void AnalysisConfigGUI::on_activate_common_mode(GtkWidget *w, AnalysisConfigGUI *A)
{

}

void AnalysisConfigGUI::auto_connect()
{
    SET_CONNECT(on_activate_common_mode);
}

void AnalysisConfigGUI::set_values()
{
    _cnf.set_weights(entry2double("txtPedWeight"), entry2double("txtNoiseWeight"));
    _cnf.seed_cut(entry2double("txtSeedCut"));
    _cnf.neigh_cut(entry2double("txtNeighCut"));
    _cnf.sn_cut(entry2double("txtSNCut"));
    _cnf.do_cmmd(btn_active("btnCmmdActive"));
}

void PluginConfigGUI::auto_connect()
{
    SET_CONNECT(on_browse_lib);
    SET_CONNECT(on_browse_path);
    SET_CONNECT(on_find_symbols);
}

void PluginConfigGUI::on_find_symbols(GtkWidget *, PluginConfigGUI *P)
{
    P->set_values();

    char *libname = P->_cnf.lib_path();
    PluginHook *ph = new PluginHook(libname);
    ph->init( P->get_file_name(), _("hook_dialog"));
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        gchar *dirname, *basename;
        const char *filename=ph->lib().c_str();
        const char *hook;
        dirname = g_path_get_dirname(filename);
        basename = g_path_get_basename(filename);
        hook = ph->hook().c_str();
        P->_cnf.lib(basename);
        P->_cnf.path(dirname);
        P->_cnf.hook(hook);
        P->user_init();
        g_free(dirname);
        g_free(basename);
    }
    g_free(libname);
    delete ph;
}

void PluginConfigGUI::on_browse_lib(GtkWidget *, PluginConfigGUI *ps)
{
#ifdef G_OS_WIN32
    const char *filters[] = {"C/C++ libs|*.dll", "Python scripts|*.py","All|*", 0};
#else
    const char *filters[] = {"C/C++ libs|*.so", "Python scripts|*.py","All|*", 0};
#endif
    std::string ofile = ::get_file_name(_("Find shared library"), a_OpenFile, ps->_cnf.path().c_str(), filters);
    if (!ofile.empty())
    {
        gchar *dirname, *basename;
        const char *filename = ofile.c_str();
        dirname = g_path_get_dirname(filename);
        basename = g_path_get_basename(filename);
        ps->set_text("txtLibName", basename);
        ps->set_text("txtLibPath", dirname);
        ps->_cnf.path(dirname);
        ps->_cnf.lib(basename);
        if (g_str_has_suffix(basename,".py"))
        {
            ps->_cnf.lang("python");
            ps->activate_button("btnPython", true);
        }
        else if  (g_str_has_suffix(basename,".so"))
        {
            ps->_cnf.lang("c++");
            ps->activate_button("btnCpp", true);
        }
        g_free(basename);
        g_free(dirname);
    }
}

void PluginConfigGUI::on_browse_path(GtkWidget *, PluginConfigGUI *ps)
{
    std::string ofile = ::get_file_name(_("Find shared library folder"), a_OpenFolder, 0, 0);
    if (!ofile.empty())
    {
        ps->set_text("txtLibPath", ofile);
        ps->_cnf.path(ofile);
    }
}

void PluginConfigGUI::user_init()
{
    set_text("txtLibPath", _cnf.path());
    set_text("txtLibName", _cnf.lib());
    set_text("txtHook", _cnf.hook());
    if ( _cnf.lang() == "python" )
    {
        activate_button("btnPython", true);
    }
    else
        activate_button("btnCpp", true);
}

void PluginConfigGUI::set_values()
{
    _cnf.path( get_text("txtLibPath") );
    _cnf.lib( get_text("txtLibName") );
    _cnf.hook( get_text("txtHook"));
    if ( btn_active("btnPython") )
    {
        _cnf.lang( "python" );
    }
    else
    {
        _cnf.lang("c++");
    }
}
