#include <glib.h>
#ifdef HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#else
#warning "NO config"
#endif
#if defined(HAS_PYTHON)
#include <Python.h>
#endif
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>
#include <gtkhst/histogram.h>
#include <gtkhst/profile.h>
#include <gtkhst/tracer.h>
#include <gtkhst/gtkhistogram.h>
#include <gtkhst/gtkhst-compat.h>

#include "GtkUtils.h"
#include "utils.h"
#include "AlibavaGUI.h"
#include "AsciiDataFile.h"
#include "RunManager.h"
#include "ConfigFactory.h"
#include "alibava_logo.h"


#include "AlibavaMonitor.h"
AlibavaMonitor __monitor__;

#ifdef __APPLE__
#include <gtkosxapplication.h>
extern GtkosxApplication *theMacApp;
#endif

static gboolean change_daq_button(AlibavaGUI *A);
std::string get_file_name(const char *title, gboolean for_open, const char **filters);
const char *get_glade_file();



void change_button_pixmap(GtkWidget *btn, GtkWidget *img)
{
    g_return_if_fail( GTK_IS_BUTTON(btn) );
    if ( img == NULL )
        return;

    GtkContainer *bin = GTK_CONTAINER(btn);
    GList *children;
    GtkWidget *child;
    bool done = false;
    while (!done)
    {
        children = gtk_container_get_children(bin);
        for ( ; children; children = children->next )
        {
            child = GTK_WIDGET(children->data);
            if ( GTK_IS_CONTAINER(child) )
            {
                bin = GTK_CONTAINER(child);
                break;
            }
            else if ( GTK_IS_IMAGE(child) )
            {
                gtk_container_remove (GTK_CONTAINER(bin), child);
                gtk_container_add(GTK_CONTAINER(bin), img);
                done = true;
                break;
            }
        }
    }
    gtk_widget_show_all(btn);
}



AlibavaGUI::AlibavaGUI(int nn, Alibava *B) :
    DAQmanager(nn, B, RunManager::Pedestal),
    _thread(0),
    current_hst(0), temp_hst(0), _show_time_profile(false)
{
    set_monitor(this);

    // init threads
    if (!g_thread_supported())
        g_thread_init(NULL);

    _timer = g_timer_new();
    add_path(DATA_DIR);

    __monitor__.setData( this );
}

AlibavaGUI::~AlibavaGUI()
{
    save_config("alibava_last.ini");
}


Analysis &AlibavaGUI::analysis()
{
    ConfigFactory &cfg = ConfigFactory::instance();
    Analysis *ana = dynamic_cast<Analysis *>(cfg.get("analysis"));
    return *ana;
}

void AlibavaGUI::nevts(int x)
{
    ConfigFactory &cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
    daq->nevts(x);
}
void AlibavaGUI::sample_size(int x)
{
    ConfigFactory &cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
    daq->sample_size(x);
}


//void AlibavaGUI::set_run_manager( RunManager::RunType type)
//{
//    if (type == RunManager::Unknown)
//        return;
//
//    if (rm)
//        delete rm;
//
//    rm = RunManager::factory(type, &_board);
//    if (rm)
//    {
//        ConfigFactory &cfg = ConfigFactory::instance();
//        PluginConfig *plg = dynamic_cast<PluginConfig *>(cfg.get("plugin"));
//        rm->set_gui( const_cast<AlibavaGUI *>(this) );
//        if ( plg->active())
//            rm->set_plugin(plg->plugin());
//    }
//}

gboolean AlibavaGUI::send_message(AlibavaGUI *A)
{
    if ( A->_msg.empty() )
        return FALSE;

    GtkStatusbar *w= GTK_STATUSBAR(A->get_widget("statusbar"));
    gtk_statusbar_pop(w, A->_cntxt);


    gtk_statusbar_push(w, A->_cntxt, A->_msg.front().c_str());
    A->_msg.pop_front();

    return FALSE;
}

gboolean AlibavaGUI::update_labels(AlibavaGUI *A)
{

    GtkWidget *w = A->get_widget("progressbar");
    if (w)
    {
        double frac = A->get_fraction_received();
        if (frac>1.)
            frac=1.;
        if (frac<0.0)
            frac=0.0;

        __monitor__.setFraction( 100.0*frac );
        gtk_progress_bar_update(GTK_PROGRESS_BAR(w), frac);
    }
    A->int2entry("txtNtrig", A->get_received());
    __monitor__.setNtrigger(A->get_received());
    __monitor__.setNexpected(A->get_expected());
    // elapsed time
    std::ostringstream ostr;
    int tt, seconds, minutes, hours, days;
    double tm = g_timer_elapsed(A->_timer, NULL);

    tt = (int)tm;
    seconds = tt % 60;
    minutes = (tt / 60) % 60;
    days = tt / 86400;
    hours = (tt / 3600 ) % 3600 - 24*days;

    ostr << days << " " << std::setfill(' ') << std::setw(2) << hours << ":"
            << std::setfill('0') << std::setw(2) << minutes << ":" << std::setfill('0')
            << std::setw(2) << seconds;

    __monitor__.setTimeElapsed( ostr.str() );
    gtk_label_set_text(GTK_LABEL(A->get_widget("txtRunDuration")), __monitor__.getTimeElapsed().c_str());

    // rate
    double rate = ((double)A->get_received())/tm;
    std::ostringstream orate;
    orate << std::setiosflags(std::ios::fixed) << std::setw(6) << std::setprecision(1)
            << rate << " Hz";

    __monitor__.setRate( orate.str() );
    __monitor__.setRate( rate );
    gtk_label_set_text(GTK_LABEL(A->get_widget("txtRate")), __monitor__.getRate().c_str());


    if (A->get_board()->firmware()==0)
    {
        gtk_widget_set_sensitive( A->get_widget("btnHeader0"), 0 );
        gtk_widget_set_sensitive( A->get_widget("btnHeader1"), 0 );
    }




    return A->running() ? TRUE : FALSE;
}

void AlibavaGUI::set_debug(bool x)
{
    get_board()->set_debug(x);
}

//int AlibavaGUI::open(const char *fnam)
//{
//    _board.open(fnam);
//    if (!_board)
//    {
//        err_msg( "Something failed when opening the device" );
//        return 1;
//    }
//
//    // reset the card
//    if (reset()<0)
//        return -1;
//
//    return 0;
//}
//int AlibavaGUI::reset()
//{
//    int rc;
//    write_msg("*** Reseting the board\n");
//    // First thing we do is to send a reset
//    rc = _board.software_reset();
//    if (rc)
//        return -1;
//
//    return send_config();
//}
//
//int AlibavaGUI::send_config()
//{
//    // Now we send the configuration
//
//    // Configure the trigger as well
//    if (config_trigger()<0)
//        return -1;
//
//    // Configure beetle
//    if (config_beetle()<0)
//        return -1;
//
//    return 0;
//}
//
//int AlibavaGUI::config_beetle()
//{
//    return ConfigFactory::instance().config_beetle(_board);
//}
//
//int AlibavaGUI::config_trigger()
//{
//    return ConfigFactory::instance().config_trigger(_board);
//}



void AlibavaGUI::user_init()
{
    GtkScrolledWindow *scroll = GTK_SCROLLED_WINDOW(get_widget("scrolledwindow"));
    GtkTextView *txt_view = GTK_TEXT_VIEW(get_widget("txtMessageLog"));
    GtkAdjustment *txt_hadj = gtk_scrolled_window_get_vadjustment(scroll);
    set_text_buffer( (char *)txt_view, (char *)txt_hadj);

    g_object_set_data(G_OBJECT(get_widget("btnCalibration")), "type",(gpointer)RunManager::Calibration);
    g_object_set_data(G_OBJECT(get_widget("btnLaserSync")), "type", (gpointer)RunManager::LaserSync);
    g_object_set_data(G_OBJECT(get_widget("btnLaser")), "type", (gpointer)RunManager::Laser);
    g_object_set_data(G_OBJECT(get_widget("btnRadSource")), "type", (gpointer)RunManager::RadSource);
    g_object_set_data(G_OBJECT(get_widget("btnPedestals")),"type", (gpointer)RunManager::Pedestal);

    ConfigFactory &cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
    LaserConfig *laser = dynamic_cast<LaserConfig *>(cfg.get("laser"));

    int2entry("entryMaxEvents", daq->nevts());
    int2entry("txtDelay", laser->delay() );
    click_button("btnPedestals");
    click_button("btnSignal");
    click_button("btnRawData");
    create_signal_histogram();

    _cntxt =
        gtk_statusbar_get_context_id(GTK_STATUSBAR(get_widget("statusbar")),
                                     "AlibavaDAQ");




#ifdef MAC_INTEGRATION
    gtkosx_application_set_use_quartz_accelerators(theMacApp, TRUE);
    GtkWidget *item,*quit_item, *preferences, *about_item, *menubar = get_widget("menubar");


    gtk_widget_hide (menubar);
    gtkosx_application_set_menu_bar(theMacApp, GTK_MENU_SHELL(menubar));

    item=get_widget("separatormenuitem1");
    about_item = get_widget("about");
    quit_item = get_widget("quit");
    preferences = get_widget("settings");
    gtk_widget_hide(quit_item);
    gtk_widget_hide(item);

    gtk_menu_item_set_label(GTK_MENU_ITEM(preferences), "Preferences");

    gtkosx_application_insert_app_menu_item  (theMacApp,
                                              about_item, 1);
    gtkosx_application_insert_app_menu_item  (theMacApp,
                                              preferences, 2);


//     group = gtk_osxapplication_add_app_menu_group (theApp);
//     gtk_osxapplication_add_app_menu_item  (theApp, group,
//                                            GTK_MENU_ITEM (preferences_item));

    gtkosx_application_sync_menubar (theMacApp);
    gtkosx_application_ready(theMacApp);
#endif


    GtkWidget *ntb = get_widget("notebookMonitor");
    gtk_widget_show_all(ntb);

}

void AlibavaGUI::set_handlers()
{
    handlers["create_histogram"] = (GladeHandler)create_histogram;
}

void AlibavaGUI::auto_connect()
{
    SET_CONNECT(on_open);
    SET_CONNECT(on_save);
    SET_CONNECT(on_save_as);
    SET_CONNECT(on_save_pedestals);
    SET_CONNECT(on_load_pedestals);
    SET_CONNECT(on_about);
    SET_CONNECT(on_buttonDAQ);
    SET_CONNECT(on_runtype);
    SET_CONNECT(on_calibration_config);
    SET_CONNECT(on_lasersync_config);
    SET_CONNECT(on_beetle_config);
    SET_CONNECT(on_reset);
    SET_CONNECT(on_laser_config);
    SET_CONNECT(on_trigger_config);
    SET_CONNECT(on_log_data);
    SET_CONNECT(on_daq_config);
    SET_CONNECT(on_analysis_config);
    SET_CONNECT(on_data_type);
    SET_CONNECT(on_signal_select);
    SET_CONNECT(on_time_profile);
    SET_CONNECT(on_reset_hst);
    SET_CONNECT(on_help_activate);
    SET_CONNECT(on_plugin_activate);
    SET_CONNECT(on_plugin_menu_activate);
}

void AlibavaGUI::process_text(const std::string &s, GtkWidget *w)
{
    if (s=="txtDelay")
    {
        LaserConfig *laser = dynamic_cast<LaserConfig *>(ConfigFactory::instance().get("laser"));
        laser->delay(entry2int(s) );
    }
    else if (s=="entryMaxEvents")
    {
        DAQConfig *daq = dynamic_cast<DAQConfig *>(ConfigFactory::instance().get("daq"));
        daq->nevts(entry2int(s));
        nevts(daq->nevts());
    }
}

void AlibavaGUI::add_message(const char *msg)
{
    if (!msg || ! *msg)
        return;

    _msg.push_back(msg);
    write_msg("%s\n", msg);
    g_idle_add( (GSourceFunc)send_message, this);
}

void AlibavaGUI::on_reset(GtkWidget *, AlibavaGUI *A)
{
    A->reset();
}

void AlibavaGUI::on_daq_config(GtkWidget *, AlibavaGUI *A)
{
    int rc;
    DAQConfig *daq = dynamic_cast<DAQConfig *>(ConfigFactory::instance().get("daq"));

    DAQConfigGUI *cnf = new DAQConfigGUI(*daq, A->get_board()->firmware());
    cnf->init(get_glade_file(), "DAQDialog");
    rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
    if (rc!=GTK_RESPONSE_CANCEL)
    {
        cnf->set_values();
        *daq = cnf->config();
        A->get_board()->delay( daq->delay());
        A->int2entry("entryMaxEvents", daq->nevts());
    }
    delete cnf;
}

void AlibavaGUI::on_laser_config(GtkWidget *, AlibavaGUI *A)
{
    int rc;
    LaserConfig *laser = dynamic_cast<LaserConfig *>(ConfigFactory::instance().get("laser"));
    LaserConfigGUI *cnf = new LaserConfigGUI(*laser, A->get_board()->firmware());
    cnf->init(get_glade_file(), "LaserConfigDialog");
    rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
    if (rc!=GTK_RESPONSE_CANCEL)
    {
        cnf->set_values();
        *laser = cnf->config();
    }
    delete cnf;
}

void AlibavaGUI::on_trigger_config(GtkWidget *, AlibavaGUI *A)
{
    int rc;
    TriggerConfig *tc = dynamic_cast<TriggerConfig *>(ConfigFactory::instance().get("trigger"));
    TriggerConfigGUI *cnf = new TriggerConfigGUI(*tc, A->get_board()->firmware());
    cnf->init(get_glade_file(), "RSDialog");
    do
    {
        rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
        if (rc!=GTK_RESPONSE_CANCEL)
        {
            cnf->set_values();
            *tc = cnf->config();
            ConfigFactory::instance().config_trigger(*A->get_board());
        }
    } while (rc==GTK_RESPONSE_APPLY);
    delete cnf;
}

void AlibavaGUI::on_beetle_config(GtkWidget *, AlibavaGUI *A)
{
    int rc;
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance().get("beetle"));

    BeetleConfigGUI *cnf = new BeetleConfigGUI(*beetle, A->get_board()->firmware());
    cnf->init(get_glade_file(), "BConfigDialog");
    do
    {
        rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
        if (rc!=GTK_RESPONSE_CANCEL)
        {
            cnf->set_values();
            *beetle = cnf->config();
            ConfigFactory::instance().config_beetle(*A->get_board());
        }
    } while (rc == GTK_RESPONSE_APPLY);
    delete cnf;
}

void AlibavaGUI::on_calibration_config(GtkWidget *w, AlibavaGUI *A)
{
    int rc;
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(ConfigFactory::instance().get("calibration"));
    CalibrationGUI *cnf = new CalibrationGUI(*cal, A->get_board()->firmware());
    cnf->init(get_glade_file(), "CalibDialog");

    while (true)
    {
        rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
        if (rc!=GTK_RESPONSE_CANCEL)
        {
            *cal = cnf->config();
            if ( rc == GTK_RESPONSE_OK )
                break;
        }
        else
            break;
    }
    delete cnf;
}

void AlibavaGUI::on_lasersync_config(GtkWidget *w, AlibavaGUI *A)
{
    int rc;
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(ConfigFactory::instance().get("lasersync"));
    LaserSyncGUI *cnf = new LaserSyncGUI(*lc, A->get_board()->firmware());
    cnf->init(get_glade_file(), "LaserDialog");

    while (true)
    {
        rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
        if (rc!=GTK_RESPONSE_CANCEL)
        {
            cnf->set_values();
            *lc  = cnf->config();
            if ( rc == GTK_RESPONSE_OK )
                break;
        }
        else
            break;
    }
    delete cnf;

}

void AlibavaGUI::on_analysis_config(GtkWidget *w, AlibavaGUI *A)
{
    int rc;
    Analysis *ana = dynamic_cast<Analysis *>(ConfigFactory::instance().get("analysis"));
    AnalysisConfigGUI *cnf = new AnalysisConfigGUI(*ana, A->get_board()->firmware());
    cnf->init(get_glade_file(), "AnalysisDialog");

    while (true)
    {
        rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
        if (rc!=GTK_RESPONSE_CANCEL)
        {
            cnf->set_values();
            ana->cpy_data(cnf->config());
            if ( rc == GTK_RESPONSE_OK )
                break;
        }
        else
            break;
    }
    delete cnf;
}

static GtkHst1D* get_hst_from_frame(GtkWidget *frame)
{
    if (!frame)
        return 0;

    gpointer data = g_object_get_data(G_OBJECT(frame), "hst");
    if (!data)
        return 0;

    if (!GTK_IS_HST1D(data))
        return 0;

    return GTK_HST1D(data);
}

void fill_signal_histogram(GtkWidget *frame, double val, int ichip)
{
    GtkHst1D *ghst = get_hst_from_frame(frame );
    if (!ghst)
        return;

    HstHistogram *hst = HST_HISTOGRAM(g_object_get_data(G_OBJECT(ghst), "hAll"));
    hst_histogram_fill(hst, val, 1.);

    if (ichip==0)
        hst = HST_HISTOGRAM(g_object_get_data(G_OBJECT(ghst), "hChip1"));
    else
        hst = HST_HISTOGRAM(g_object_get_data(G_OBJECT(ghst), "hChip2"));

    hst_histogram_fill(hst, val, 1.);
}

void fill_time_profile(GtkWidget *frame, double val, double tim, int ichip)
{
    GtkHst1D *ghst = get_hst_from_frame(frame );
    if (!ghst)
        return;

    HstProfile *hst = HST_PROFILE(g_object_get_data(G_OBJECT(ghst), "pAll"));
    if (hst)
        hst_profile_fill(hst, tim, val, 1.);

    if (ichip==0)
        hst = HST_PROFILE(g_object_get_data(G_OBJECT(ghst), "pChip1"));
    else
        hst = HST_PROFILE(g_object_get_data(G_OBJECT(ghst), "pChip2"));

    if (hst)
        hst_profile_fill(hst, tim, val, 1.);
}


/**
 * Removes existing histograms
 *
 */
void AlibavaGUI::unref_histograms(GObject *ghst)
{
    const char *hnam[] =
        {
         "hAll", "hChip1", "hChip2",
         "pAll", "pChip1", "pChip2",
         0
        };
    int ihst;
    for (ihst=0; hnam[ihst];ihst++)
    {
        gpointer obj;
        obj = g_object_get_data(ghst, hnam[ihst]);
        if (obj)
            g_object_unref(G_OBJECT(obj));
    }

}

void AlibavaGUI::create_time_profiles(GObject *ghst)
{
    HstProfile *hprof;
    hprof = hst_profile_new(50, 0., 100., 0);
    g_object_set_data(G_OBJECT(ghst),"pAll", hprof);

    hprof = hst_profile_new(50, 0., 100., 0);
    g_object_set_data(G_OBJECT(ghst),"pChip1", hprof);


    hprof = hst_profile_new(50, 0., 100., 0);
    g_object_set_data(G_OBJECT(ghst),"pChip2", hprof);

}


/**
 * create_signal_histogram:
 *
 * creates the signal histograms when a new run starts.
 *
 */
void AlibavaGUI::create_signal_histogram()
{
    GtkHst1D *ghst = get_hst_from_frame(get_widget("hstSignal") );
    if (!ghst)
        return;

    /*
     * Set the proper title
     */
    g_object_set(G_OBJECT(ghst),
                 "ytitle", _("no. of entries"),
                 "xtitle", _("Signal (ADC)"),
                 NULL);

    /*
     * Delete existing histograms
     */
    HstHistogram *hst;


    unref_histograms(G_OBJECT(ghst));
    /*
     * Create new histograms
     */
    hst = hst_histogram_new(256, -1023.5, 1023.5, 0);
    g_object_set(G_OBJECT(ghst), "func", hst, NULL);

    g_object_set_data(G_OBJECT(ghst),"hAll", hst);
    current_hst = HST_HISTOGRAM(hst);

    hst = hst_histogram_new(256, -1023.5, 1023.5, 0);
    g_object_set_data(G_OBJECT(ghst),"hChip1", hst);

    hst = hst_histogram_new(256, -1023.5, 1023.5, 0);
    g_object_set_data(G_OBJECT(ghst),"hChip2", hst);


    create_time_profiles(G_OBJECT(ghst));

}

void AlibavaGUI::create_signal_profile(const char *tit, int nb, double xmin, double xmax)
{
    GtkHst1D *ghst = get_hst_from_frame(get_widget("hstSignal") );
    if (!ghst)
        return;

    /*
     * Set proper titles
     */
    g_object_set(G_OBJECT(ghst),
                 "ytitle", _("average signal (ADC)"),
                 "xtitle", tit,
                 NULL);

    /*
     * Delete existing histograms
     */
    unref_histograms(G_OBJECT(ghst));

    /*
     * Fill
     */
    HstProfile *hst;
    double step = (xmax-xmin)/nb;
    hst = hst_profile_new(nb, xmin-step/2., xmax-step/2., 0);
    g_object_set(G_OBJECT(ghst), "func", hst, NULL);
    g_object_set_data(G_OBJECT(ghst),"hAll", hst);
    current_hst = HST_HISTOGRAM(hst);

    hst = hst_profile_new(nb, xmin-step/2., xmax-step/2., 0);
    g_object_set_data(G_OBJECT(ghst),"hChip1", hst);

    hst = hst_profile_new(nb, xmin-step/2., xmax-step/2., 0);
    g_object_set_data(G_OBJECT(ghst),"hChip2", hst);

    create_time_profiles(G_OBJECT(ghst));

    gtk_widget_show_all(get_widget("hstSignal"));
    gtk_widget_queue_draw( get_widget("hstSignal"));
}

void AlibavaGUI::reset_hst()
{
    GtkHst1D *ghst = get_hst_from_frame(get_widget("hstSignal") );
    if (!ghst)
        return;

    const char *hnam[] =
        {
         "hAll", "hChip1", "hChip2",
         "pAll", "pChip1", "pChip2",
         0
        };
    int ihst;
    for (ihst=0; hnam[ihst];ihst++)
    {
        gpointer obj;
        obj = g_object_get_data(G_OBJECT(ghst), hnam[ihst]);
        if (obj && HST_IS_HISTOGRAM(obj))
            hst_histogram_reset(HST_HISTOGRAM(obj));
    }
}

/**
 * Here we create a widget...
 */
GtkWidget * AlibavaGUI::create_histogram(AlibavaGUI *wnd, char *name, char *s1, char *s2,
                                         int i1, int i2)
{
    ConfigFactory &cfg = ConfigFactory::instance();
    Analysis *ana = dynamic_cast<Analysis *>(cfg.get("analysis"));

    std::string hnam = name;
    GtkWidget *frame, *w;
    HstHistogram *hst;
    frame = gtk_frame_new(NULL);
    if (hnam=="hstSignal")
    {
        hst = hst_histogram_new(256, -1023.5, 1023.5, 0);
        w = gtk_hst1d_new(HST_FUNCTION(hst));
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_xtitle(GTK_HST1D(w), _("Signal (ADC)"));
        gtk_hst1d_set_colors(GTK_HST1D(w), "background", "white", NULL);
    }
    else if (hnam=="hstPedestal")
    {
        hst = hst_histogram_new(256, 0., 256., 0);
        g_object_set(G_OBJECT(hst), "ptr", ana->pedestals(), NULL);
        w = gtk_hst1d_new(HST_FUNCTION(hst));
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), _("Pedestals (ADC)"));
        gtk_hst1d_set_xtitle(GTK_HST1D(w), _("Channel"));
        gtk_hst1d_set_colors(GTK_HST1D(w), "background", "white", NULL);
    }
    else if (hnam=="hstNoise")
    {
        hst = hst_histogram_new(256, 0., 256., 0);
        g_object_set(G_OBJECT(hst), "ptr", ana->noise(), NULL);
        w = gtk_hst1d_new(HST_FUNCTION(hst));
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), _("Noise (ADC)"));
        gtk_hst1d_set_xtitle(GTK_HST1D(w), _("Channel"));
        gtk_hst1d_set_colors(GTK_HST1D(w), "background", "white", NULL);

    }
    else if (hnam=="hstHitmap")
    {
        hst = hst_histogram_new(256, 0., 256., ana->hitmap());
        w = gtk_hst1d_new(HST_FUNCTION(hst));
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_xtitle(GTK_HST1D(w), _("Channel"));
        gtk_hst1d_set_ytitle(GTK_HST1D(w), _("# of entries"));
        gtk_hst1d_set_colors(GTK_HST1D(w), "background", "white", NULL);
        wnd->hitmap_hst = hst;
    }
    else if (hnam=="hstTemp")
    {
        HstTracer *tracer = hst_tracer_new(50);
        w = gtk_hst1d_new(HST_FUNCTION(tracer));
        gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);
        gtk_hst1d_set_colors(GTK_HST1D(w), "foreground", "firebrick", "background",
        "black", "axis", "darkseagreen", NULL);
        gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), "Temperature");
        wnd->temp_hst = tracer;
    }
    else if (hnam=="hstTime")
    {
        HstTracer *tracer = hst_tracer_new(50);
        w = gtk_hst1d_new(HST_FUNCTION(tracer));
        gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);
        gtk_hst1d_set_colors(GTK_HST1D(w), "foreground", "firebrick", "background",
        "black", "axis", "darkseagreen", NULL);
        gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), "Time");
        wnd->time_hst = tracer;
    }
    else if (hnam == "hstEvent")
    {
        hst = hst_histogram_new(256, 0., 256., ana->raw());
        w = gtk_hst1d_new(HST_FUNCTION(hst));
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_draw_x_axis(GTK_HST1D(w), HST_AXIS_NONE);
        gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), "Signal (ADC)");
        g_object_set(G_OBJECT(w), "hymin", 0., "hymax", 1024., NULL);
        gtk_hst1d_set_colors(GTK_HST1D(w),
                             "foreground", "firebrick",
                             "background", "black",
                             "axis", "darkseagreen",
                             NULL);
        g_object_set_data(G_OBJECT(w), "raw", hst);
        g_object_set_data(G_OBJECT(w), "signal", hst_histogram_new(256, 0., 256., ana->signal()));
        g_object_set_data(G_OBJECT(w), "header0", hst_histogram_new(16, -0.5, 15.5, ana->header(0)));
        g_object_set_data(G_OBJECT(w), "header1", hst_histogram_new(16, -0.5, 15.5, ana->header(1)));

        wnd->event_hst = hst;
    }
    else if (hnam == "hstCmmdNoise")
    {
        wnd->noise_tracer[0] = hst_tracer_new(50);
        wnd->noise_tracer[1] = hst_tracer_new(50);
        hst_function_set_name(HST_FUNCTION(wnd->noise_tracer[0]), (gchar *)"chip 1");
        hst_function_set_name(HST_FUNCTION(wnd->noise_tracer[1]), (gchar *)"chip 2");
        w = gtk_hst1d_new(HST_FUNCTION(wnd->noise_tracer[0]));
        gtk_hst1d_add_curve(GTK_HST1D(w), HST_FUNCTION(wnd->noise_tracer[1]));
        gtk_hst1d_set_curve_colors(GTK_HST1D(w), "medium sea green", "royal blue", "firebrick", "gold", "turquoise",NULL);
        gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);
        gtk_hst1d_set_colors(GTK_HST1D(w), "foreground", "firebrick", "background", "black", "axis", "darkseagreen", NULL);
        gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), _("Noise (ADC)"));
        g_object_set(G_OBJECT(w), "hymin", 0., "hymax", 10., NULL);
    }
    else if (hnam == "hstCmmd")
    {
        wnd->cmmd_tracer[0] = hst_tracer_new(50);
        wnd->cmmd_tracer[1] = hst_tracer_new(50);
        hst_function_set_name(HST_FUNCTION(wnd->cmmd_tracer[0]), (gchar *)"chip 1");
        hst_function_set_name(HST_FUNCTION(wnd->cmmd_tracer[1]), (gchar *)"chip 2");
        w = gtk_hst1d_new(HST_FUNCTION(wnd->cmmd_tracer[0]));
        gtk_hst1d_add_curve(GTK_HST1D(w), HST_FUNCTION(wnd->cmmd_tracer[1]));
        gtk_hst1d_set_curve_colors(GTK_HST1D(w), "medium sea green", "royal blue", "firebrick", "gold", "turquoise",NULL);
        gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);
        gtk_hst1d_set_colors(GTK_HST1D(w), "foreground", "firebrick", "background","black", "axis", "darkseagreen", NULL);
        gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
        gtk_hst1d_set_ytitle(GTK_HST1D(w), _("Common Mode (ADC)"));
    }
    else
        w = gtk_label_new("X");

    if (GTK_IS_HST1D(w))
    {
        gtk_hst1d_monitor_start(GTK_HST1D(w), 500);
    }
    g_object_set(G_OBJECT(frame), "shadow-type", GTK_SHADOW_IN, NULL);
    g_object_set_data(G_OBJECT(frame), "hst", w);
    wnd->hst_list[hnam] = w;
    gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(w));
    gtk_widget_show_all(frame);
    return frame;

}

void AlibavaGUI::on_log_data(GtkWidget *, AlibavaGUI *A)
{
    const char *filters[] = { "Root files|*.root", "All|*", 0 };
    std::string fnam =:: get_file_name(_("Output File"), a_SaveFile, 0, filters);
    if (!fnam.empty())
    {
        A->set_data_file(fnam);
    }
}

void AlibavaGUI::on_help_activate(GtkWidget *w, AlibavaGUI *A)
{
    gtk_help_display(NULL);
}
void AlibavaGUI::on_plugin_menu_activate(GtkWidget *w, AlibavaGUI *A)
{
    int rc;
    ConfigFactory &cfg = ConfigFactory::instance();
    PluginConfig *plg = dynamic_cast<PluginConfig *>(cfg.get("plugin"));
    PluginConfigGUI *cnf = new PluginConfigGUI( *plg , A->get_board()->firmware());
    cnf->init(get_glade_file(),"PluginDialog");
    rc = gtk_dialog_run(GTK_DIALOG(cnf->get_main_window()));
    if (rc!=GTK_RESPONSE_CANCEL)
    {
        cnf->set_values();
        *plg = cnf->config();
        on_plugin_activate( A->get_widget("btnPlugin"), A);
    }
    delete cnf;
}
void AlibavaGUI::on_plugin_activate(GtkWidget *w, AlibavaGUI *A)
{
    GtkWidget *image;
    ConfigFactory &cfg = ConfigFactory::instance();
    PluginConfig *plg = dynamic_cast<PluginConfig *>(cfg.get("plugin"));

    bool active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    if ( active )
    {
        image = gtk_image_new_from_stock ("gtk-connect", GTK_ICON_SIZE_BUTTON);
    }
    else
    {
        image = gtk_image_new_from_stock ("gtk-disconnect", GTK_ICON_SIZE_BUTTON);
    }
    plg->active(active);
    change_button_pixmap(w, image);
    gtk_widget_queue_draw(w);
}

void AlibavaGUI::on_open(GtkWidget *, AlibavaGUI *A)
{
    const char *filters[] = { "Config files|*.ini", "All|*", 0 };
    std::string ofile =:: get_file_name(_("Load config"), a_OpenFile, 0, filters);
    if (!ofile.empty())
    {
        A->load_config( ofile.c_str() );
    }
}

void AlibavaGUI::on_save_as(GtkWidget *w, AlibavaGUI *A)
{
    const char *filters[] = { "Config files|*.ini", "All|*", 0 };
    A->sfile = ::get_file_name(_("Save config"), a_SaveFile, 0, filters);
    if (!A->sfile.empty())
        on_save(w,A);
}

void AlibavaGUI::on_save_pedestals(GtkWidget *w, AlibavaGUI *A)
{
    const char *filters[] = { "Pedestal files|*.ped", "All|*", 0 };
    std::string ped_file = ::get_file_name(_("Save Pedestals"), a_SaveFile, 0, filters);
    if (!ped_file.empty())
    {
        std::ofstream ofile(ped_file.c_str());
        if (!ofile)
        {
            err_msg(_("could not open pedestal file %s"), ped_file.c_str());
            return;
        }
        for (int i=0;i<256;i++)
        {
            ofile << A->analysis().ped(i) << '\t' << A->analysis().noise(i) << std::endl;
        }
        write_msg(_("Pedestals saved in %s\n"), ped_file.c_str());
        ofile.close();
    }
}

void AlibavaGUI::on_load_pedestals(GtkWidget *, AlibavaGUI *A)
{
    const char *filters[] = { "Pedestal files|*.ped", "All|*", 0 };
    std::string ped_file = ::get_file_name(_("Load Pedestals"), a_OpenFile, 0, filters);
    if (!ped_file.empty())
    {
        std::ifstream ofile(ped_file.c_str());
        if (!ofile)
        {
            err_msg(_("could not open pedestal file %s"), ped_file.c_str());
            return;
        }
        for (int i=0;i<256;i++)
        {
            double p, n;
            if (ofile.eof())
                break;

            ofile >> std::ws >> p >> std::ws >> n >> std::ws;

            if (!ofile.fail())
            {
                A->analysis().ped(i, p);
                A->analysis().noise(i, n);
            }
        }
        write_msg(_("Pedestals loaded from %s\n"), ped_file.c_str());
        ofile.close();
    }
}

void AlibavaGUI::on_save(GtkWidget *w, AlibavaGUI *A)
{
    if (A->sfile.empty())
        A->sfile="alibava_last.ini";

    A->save_config(A->sfile.c_str() );
    write_msg(_("Configuration saved in %s\n"), A->sfile.c_str());
    A->sfile.clear();

}
void AlibavaGUI::on_about(GtkWidget *W, AlibavaGUI *A)
{
    int rc;
    GtkWidget *wndAbout;
    GtkWidget *vbox, *image;
    GtkWidget *lblVersion, *lblgtk, *lbldesc, *lblweb;
    GdkPixbuf *pixbuf;

    char line[256];

    wndAbout = gtk_dialog_new_with_buttons ("Alibava Systems",
                                            GTK_WINDOW( A->get_main_window() ),
                                            (GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
                                            GTK_STOCK_OK, GTK_RESPONSE_OK,
                                            NULL
                                            );
//    gtk_window_set_title (GTK_WINDOW (wndAbout), "Alibava Systems");
//    gtk_window_set_transient_for(GTK_WINDOW(wndAbout), GTK_WINDOW(A->get_main_window())));
    gtk_window_set_position (GTK_WINDOW (wndAbout), GTK_WIN_POS_CENTER_ON_PARENT);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (wndAbout), TRUE);
    gtk_window_set_type_hint (GTK_WINDOW (wndAbout), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_dialog_set_has_separator (GTK_DIALOG (wndAbout), FALSE);

    vbox = gtk_dialog_get_content_area(GTK_DIALOG(wndAbout));

    pixbuf = gdk_pixbuf_new_from_inline(-1, alibava_logo, FALSE, NULL);
    image = gtk_image_new_from_pixbuf( pixbuf );
    gtk_box_pack_start( GTK_BOX(vbox), image,TRUE, TRUE, 0);

    sprintf(line,_("<b><big><big>alibava-gui </big></big>\n" PACKAGE_VERSION"-"PACKAGE_RELEASE  "</b>"));
    lblVersion = gtk_label_new ( line );
    gtk_box_pack_start (GTK_BOX (vbox), lblVersion, FALSE, FALSE, 0);
    gtk_label_set_use_markup (GTK_LABEL (lblVersion), TRUE);
    gtk_label_set_justify (GTK_LABEL (lblVersion), GTK_JUSTIFY_CENTER);
    gtk_misc_set_padding (GTK_MISC (lblVersion), 0, 10);

    lbldesc = gtk_label_new (_("alibava-gui is a graphical user interface that controls ALIBAVA's data acqisition"));
    gtk_widget_show (lbldesc);
    gtk_box_pack_start (GTK_BOX (vbox), lbldesc, FALSE, FALSE, 0);
    gtk_label_set_use_markup (GTK_LABEL (lbldesc), TRUE);
    gtk_label_set_justify (GTK_LABEL (lbldesc), GTK_JUSTIFY_CENTER);
    gtk_label_set_line_wrap (GTK_LABEL (lbldesc), TRUE);

    sprintf(line, _("<small>\n(Using Gtk+ %d.%d.%d-%d and GLib2 %d.%d.%d-%d)</small>"),
            GTK_MAJOR_VERSION,GTK_MINOR_VERSION, GTK_MICRO_VERSION, GTK_INTERFACE_AGE,
            GLIB_MAJOR_VERSION,GLIB_MINOR_VERSION, GLIB_MICRO_VERSION, glib_interface_age
    );
    lblgtk = gtk_label_new( line );
    gtk_box_pack_start (GTK_BOX (vbox), lblgtk, FALSE, FALSE, 0);
    gtk_label_set_use_markup (GTK_LABEL (lblgtk), TRUE);

#if  GTK_CHECK_VERSION(2,18,0)
    lblweb = gtk_label_new("\n<big><a href=\"http://www.alibavasystems.com\">www.alibavasystems.com</a></big>\n");
    gtk_label_set_use_markup (GTK_LABEL (lblweb), TRUE);
#else
    lblweb = gtk_label_new("\n<big>www.alibavasystems.com</big>\n");
#endif
    gtk_box_pack_start (GTK_BOX (vbox), lblweb, FALSE, FALSE, 0);


    gtk_widget_show_all( wndAbout );

    gtk_dialog_run( GTK_DIALOG(wndAbout) );

    gtk_widget_destroy( wndAbout );
}

void AlibavaGUI::on_data_type(GtkWidget *w, AlibavaGUI *A)
{
    GtkHst1D *ghst = get_hst_from_frame(A->get_widget("hstEvent"));
    double ymin,ymax,yreg = 300.;
    HstHistogram *hst=0;
    if (!ghst)
        return;


    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) )
    {
        std::string name = gtk_widget_get_name(w);
        if ( name == "btnRawData")
        {
            ymin = 512. - yreg;
            ymax = 512. + yreg;
            hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst),"raw" ));
        }
        else if ( name == "btnSignal" )
        {
            ymin = -yreg;
            ymax =  yreg;
            hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst),"signal" ));
        }
        else if ( name == "btnHeader0" )
        {
            ymin = 512. - yreg;
            ymax = 512. + yreg;
            hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst),"header0" ));
        }
        else if ( name == "btnHeader1")
        {
            ymin = 512. - yreg;
            ymax = 512. + yreg;
            hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst),"header1" ));
        }
        if ( hst &&  GTK_IS_HST1D(ghst))
        {
            std::string hname = name.substr(3);
            g_object_set(G_OBJECT(ghst),
                                 "func", hst,
                                 "hymin", ymin,
                                 "hymax", ymax,
                                 "ytitle", hname.c_str(),
                                 NULL);
        }
        else
            err_msg( _("... unknown data type...") );
    }
}

void AlibavaGUI::on_signal_select(GtkWidget *w, AlibavaGUI *A)
{
    GtkHst1D *ghst = get_hst_from_frame(A->get_widget("hstSignal"));
    HstHistogram *hst=0;
    if (!ghst)
        return;

    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) )
    {
        std::string name = gtk_widget_get_name(w);
        if (name=="btnAll")
        {
            if ( A->_show_time_profile)
            {
                hst = HST_HISTOGRAM(g_object_get_data(G_OBJECT(ghst),"pAll"));
            }
            else
            {
                hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst), "hAll"));
            }
        }
        else if (name=="btnChip1")
        {
            if ( A->_show_time_profile)
                hst = HST_HISTOGRAM(g_object_get_data(G_OBJECT(ghst),"pChip1"));
            else
                hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst), "hChip1"));
        }
        else
        {
            if ( A->_show_time_profile)
                hst = HST_HISTOGRAM(g_object_get_data(G_OBJECT(ghst),"pChip2"));
            else
                hst = HST_HISTOGRAM( g_object_get_data(G_OBJECT(ghst), "hChip2"));
        }
        if (hst && GTK_IS_HST1D(ghst))
        {
            g_object_set(G_OBJECT(ghst),"func", hst, NULL);
            if (A->_show_time_profile)
            {
                g_object_set(G_OBJECT(ghst),
                             "xtitle",_("Time (ns)"),
                             "ytitle",_("Average Signal (ADC)"),
                             NULL);
            }
            else
            {
                std::string x_title(_("Signal (ADC)"));
                std::string y_title(_("no. of entries"));
                CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(ConfigFactory::instance().get("calibration"));

                switch (A->get_run_type()) {
                    case RunManager::Calibration:
                        if ( cal->scan_charge() )
                            x_title = _("Injected Pulse (e)");
                        else
                            x_title = _("Delay (ns)");

                        y_title = _("Average Signal (ADC)");
                        break;
                    case RunManager::LaserSync:
                        x_title = _("Laser Delay (ns)");
                        y_title = _("Average Signal (ADC)");
                        break;
                    default:
                        break;
                }
                g_object_set(G_OBJECT(ghst),
                             "xtitle",x_title.c_str(),
                             "ytitle",y_title.c_str(),
                             NULL);
            }
        }

    }


}
void AlibavaGUI::on_time_profile( GtkWidget *w, AlibavaGUI *A)
{
    GtkWidget *frame =A->get_widget("hstSignal");
    A->_show_time_profile = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));

    GtkHst1D *ghst = get_hst_from_frame(frame );
    if (!ghst)
        return;

    GtkWidget *btn = A->get_widget("btnAll");
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)) )
    {
        on_signal_select(btn, A);
        return;
    }
    btn = A->get_widget("btnChip1");
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)) )
    {
        on_signal_select(btn, A);
        return;
    }
    btn = A->get_widget("btnChip2");
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)) )
    {
        on_signal_select(btn, A);
        return;
    }
    return;
}

int AlibavaGUI::prepare_for_run()
{
    ConfigFactory &cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
    g_idle_add( (GSourceFunc) change_daq_button, this);
    g_timeout_add( 500, (GSourceFunc)update_labels, this);

    // Set the proper label in the button
    write_msg( _("Events/sample: %d\n"), daq->sample_size());
    return DAQmanager::prepare_for_run();
}

void AlibavaGUI::on_buttonDAQ(GtkWidget *w, AlibavaGUI *A)
{
    GError *error=NULL;

    if (A->running())
    {
        A->stop(true);
        A->get_board()->stop(true);
        return;
    }
    ConfigFactory &cfg = ConfigFactory::instance();

    // This is done already in DAQmanager
//    A->running(true);
//    A->stop(false);

    A->prepare_for_run();


    /*
     * Prepare the output file (if any)
     */
    if (A->out_file())
    {
        std::ostringstream ostr;
        ostr << _("Opened output file: ") << A->out_file()->file_name();
        A->add_message(ostr.str().c_str());
    }
    else
    {
        A->add_message(_("Run Started"));
    }


    on_time_profile(A->get_widget("btnTimeProfile"), A);
    A->_thread = g_thread_create((GThreadFunc)acquire_thrd, A, FALSE, &error);
}

gpointer AlibavaGUI::acquire_thrd(AlibavaGUI *A, bool is_thread)
{
    g_timer_reset(A->_timer);
    g_timer_start(A->_timer);

    int rc = A->acquire();
    if (!rc)
        A->stop_run();

    g_timer_stop(A->_timer);
//    A->running(false);
//    A->stop(false);
    g_idle_add( (GSourceFunc) change_daq_button, A);

    if (is_thread)
        g_thread_exit(0);
    return 0;
}

void AlibavaGUI::on_reset_hst(GtkWidget *, AlibavaGUI *A)
{
    A->reset_hst();
}

void AlibavaGUI::on_runtype(GtkWidget *w, AlibavaGUI *A)
{
    char line[1024];
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) )
    {
        std::ostringstream ostr;
        std::string stype = gtk_widget_get_name(w);
        sprintf(line, _("Run type changed to %s"), stype.substr(3).c_str());
        A->add_message(line);
        int itype= GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "type"));
        RunManager::RunType type = (RunManager::RunType)itype;
        if (type==RunManager::Unknown)
        {
            write_msg( _("Bad run type: %d \n"), type );
        }
        else
        {
            A->set_run_type(type);
            __monitor__.setRunType(stype.substr(3));
            __monitor__.setRunTypeVal(type);
        }
    }
}

std::string __analysis__("analysis");
std::string __daq__("daq");
std::string __plugin__("plugin");

//Receiver
static unsigned long __sync_error_counter__ = 0;

int AlibavaGUI::check_event_headers(const EventData* b)
{
    // Check for header inconsistencies
    if (get_board()->firmware() > 0)
    {
        int ih;
        int msk = get_board()->mask();
        for (ih = 0; ih < 2; ih++)
        {
            if (msk & (1 << ih))
            {
                const Chip *chip = b->chip(ih);
                double h1 = chip->header[0];
                //std::cout << "chip " << ih << " header " << h1 << std::endl;
                if (fabs(h1 - 630) > 50.0)
                {
                    return 1;
                }
            }
        }
    }
    else
    {
        int state[] = { 1, 0, 1 };
        int ichan, istate;

        for (ichan = 0; ichan < 128; ichan++)
        {
            istate = 0;

            while (true)
            {
                unsigned short value = b->chip(0)->signal[ichan + istate];
                if (state[istate])
                {
                    if (!(value > 550 && value < 650))
                        break;
                }
                else
                {
                    if (!(value > 350 && value < 450))
                        break;
                }

                istate++;
                if (istate >= 3)
                {
                    return 1;
                }
            }
        }

    }
    return 0;
}

void AlibavaGUI::process_event(const EventData *b)
{
    double x, y, t;
    static double __last_bad_header__=0.;
    int charge, delay;

    ConfigFactory &cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get(__daq__));
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg.get("calibration"));



    // temperature
    t = 0.12*b->temp() - 39.8;
    hst_histogram_fill(HST_HISTOGRAM(temp_hst), t, 1.);

    // time from TDC
//    int stop = b->time & 0xff;
//    int start = (b->time & 0xff00)>>8;
//    int coarse = (b->time & 0xff0000) >> 16;
//    double tt = 100.  - ( start+coarse-stop);
//    if (tt<0.)
//        tt = 0.;

    // Decode time from TDC and fill the histogram
    unsigned short fpart =  b->time() & 0xffff;
    short ipart = (b->time() & 0xffff0000)>>16;
    if (ipart<0)
        fpart *= -1;
    double tt = 100.*(ipart + (fpart/65535.));

    hst_histogram_fill(HST_HISTOGRAM(time_hst), tt, 1.);

    // Data
    Analysis *analysis = dynamic_cast<Analysis *>(ConfigFactory::instance().get(__analysis__));

    // Check for header inconsistencies
    if (check_event_headers(b))
    {
        double tm = g_timer_elapsed(_timer, NULL);
//        if ( tm - __last_bad_header__ > 3.3e-3 )
//        {
//            info_msg("Synchronization problem: %d", __sync_error_counter__);
//        }
        __sync_error_counter__++;
        __last_bad_header__ =  tm;
        return;
    }

    analysis->process_event((unsigned short *)b->chip(0), get_board()->mask());




    // Fill header histograms
    for (int ichip=0; ichip<2; ichip++)
    {
        const Chip *chip = b->chip(ichip);
        for (int j=0;j<16; j++)
            analysis->header(ichip)[j] = chip->header[j];
    }

    __monitor__.setRunTypeVal( get_run_type() );
    __monitor__.setValue( b->value() );


    switch (get_run_type())
    {
        case RunManager::Calibration:
            y = analysis->signal(daq->chan());
            charge = int(b->value()) & 0xff;
            delay = int(b->value()) >> 16;
            if ( cal->scan_charge())
                x = charge * 1024;
            else
                x = delay;

            hst_histogram_fill(current_hst, x, fabs(y));
            break;
        case RunManager::LaserSync:
            if (!analysis->empty())
            {
                y = analysis->hit(0).second;
                hst_histogram_fill(current_hst, b->value(), y);
                hst_histogram_fill(hitmap_hst, analysis->hit(0).first, 1.);
            }
            if (! (get_received() %10))
            {
                for (int it=0; it<2; it++)
                {
                    hst_histogram_fill(HST_HISTOGRAM(cmmd_tracer[it]),analysis->cmmd(it),1.0);
                    hst_histogram_fill(HST_HISTOGRAM(noise_tracer[it]), analysis->cmmd_noise(it),1.0);
                }
            }
            break;
        case RunManager::Laser:
        case RunManager::RadSource:
            if ( !analysis->empty() )
            {
                y = analysis->hit(0).second;
                fill_signal_histogram(get_widget("hstSignal"),y,analysis->hit(0).first/128);
                fill_time_profile(get_widget("hstSignal"),y,tt, analysis->hit(0).first/128);
                //hst_histogram_fill(current_hst, y, 1.);
                hst_histogram_fill(hitmap_hst, analysis->hit(0).first, 1.);
            }
            if (! (get_received() %10))
            {
                for (int it=0; it<2; it++)
                {
                    hst_histogram_fill(HST_HISTOGRAM(cmmd_tracer[it]),analysis->cmmd(it),1.0);
                    hst_histogram_fill(HST_HISTOGRAM(noise_tracer[it]), analysis->cmmd_noise(it),1.0);
                }
            }
            break;
        default:
            break;
    }

    PluginConfig *plg = dynamic_cast<PluginConfig *>(cfg.get(__plugin__));
    if (plg->active())
    {
        std::string data = plg->plugin()->filter_event(b);
        if (out_file() && !data.empty())
        {
            out_file()->write(DataFile::DataBlock, data.size(), data.c_str());
        }
    }
    else {
        // write data
        if (out_file())
            out_file()->write(b);
    }

}


void AlibavaGUI::getHistogram(const std::string &whst, size_t &size, char **buffer)
{
    std::map<std::string, GtkWidget *>::iterator ihst = hst_list.find(whst);
    size = 0;
    *buffer = 0;
    if (ihst == hst_list.end())
        return;

    GtkHst1D *ghst=0;
    GString *str = 0;

    *buffer = 0;
    size = 0;


    ghst = GTK_HST1D(ihst->second);

    if (!ghst)
        return;

    g_object_get(G_OBJECT(ghst),"png",&str,NULL);
    *buffer = (char *)g_malloc(str->len);
    size = str->len;
    memcpy(*buffer, str->str, str->len);
    g_string_free(str, TRUE);
}


static gboolean change_daq_button(AlibavaGUI *A)
{
    GtkWidget *w = A->get_widget("buttonDAQ");
    if (w)
    {
        gtk_button_set_label(GTK_BUTTON(w),
                             (A->running() || A->starting()) ? _("Running (click to stop)") : _("Start"));
        gtk_widget_queue_draw(w);
    }
    return FALSE;
}



