#include <algorithm>
#include <ctime>
#include <cstring>
#include "AsciiDataFile.h"
#include "utils.h"

const unsigned int AsciiDataFile::header_length = 80;
const unsigned int AsciiDataFile::data_header = 0xcafe0000;

AsciiDataFile::AsciiDataFile(const std::string &name) :
    DataFile(name), ofile(0)
{
    open(name);
}

AsciiDataFile::~AsciiDataFile()
{
    if (ofile)
    {
        ofile->close();
        delete ofile;
    }
}

void AsciiDataFile::close()
{
    if (ofile)
    {
        ofile->close();
        delete ofile;
        ofile = 0;
    }
}
void AsciiDataFile::open(const std::string &name)
{
    std::string fnam = name;
    if (fnam.empty() )
    {
        fnam = file_name();
        if (fnam.empty())
        {
            fnam = "ascii.data";
        }
    }
    ofile = new std::ofstream(fnam.c_str(), std::ios_base::binary);
    if (! (*ofile))
    {
        err_msg(_("Could not open file: %s"), fnam.c_str());
        delete ofile;
        ofile = 0;
    }
    else
    {
        // write current time
        time_t t0 = time(0);
        char cheader[header_length];
        double val = 0.;
        int i, type = 0;
        size_t hs = _header.size();
        if ( hs > header_length )
            hs = header_length;

        memset(cheader, 0, sizeof(cheader));
        memcpy(cheader, _header.c_str(), hs);
        ofile->write((const char *)&t0, sizeof(time_t));
        ofile->write((const char *)&type, sizeof(int));
        ofile->write((const char *)&header_length, sizeof(unsigned int));
        ofile->write(cheader, header_length);
        for (i=0; i<512; i++)
        {
            ofile->write((const char *)&val, sizeof(double));
        }
    }
}

void AsciiDataFile::write(const EventData *e)
{
    if (ofile)
    {
        unsigned int xxx = data_header | DataBlock;
        ofile->write((const char *)&xxx, sizeof(unsigned int));

        const EventBlock *B = e->get_block();

        xxx = sizeof(double) + sizeof(unsigned int) + 2*sizeof(struct Chip);
        double val = e->value();
        ofile->write((const char *)&xxx, sizeof(unsigned int));
        ofile->write((const char *)&val, sizeof(double));
        ofile->write((const char *)&B->time, sizeof(unsigned int));
        ofile->write((const char *)&B->temp, sizeof(unsigned short));
        ofile->write((const char *)B->chip, 2*sizeof(struct Chip));
    }
}

void AsciiDataFile::write(BlockType type, size_t size, const char *buff)
{
    if (ofile)
    {
        unsigned int xxx = data_header | UserDataBlock | type;
        unsigned int sss = size;
        ofile->write((char *)&xxx, sizeof(unsigned int));
        ofile->write((char *)&sss, sizeof(unsigned int));
        ofile->write(buff, size);
    }
}

void AsciiDataFile::write(int type, double *p, double *n)
{
    if (ofile)
    {
        char cheader[header_length];
        size_t hs = _header.size();
        if ( hs > header_length )
            hs = header_length;

        memset(cheader, 0, sizeof(cheader));
        memcpy(cheader, _header.c_str(), hs);

        std::ofstream::pos_type pos = ofile->tellp();
        ofile->seekp(sizeof(time_t));
        ofile->write((const char *)&type, sizeof(int));
        ofile->write((const char *)&header_length, sizeof(unsigned int));
        ofile->write(cheader, header_length);
        if (p && n)
        {
            ofile->write((const char *)p, 256*sizeof(double));
            ofile->write((const char *)n, 256*sizeof(double));
        }
        ofile->seekp(pos);
    }
}
