/*
 * CalibrationRun.cc
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */

#include <iostream>
#include "DAQmanager.h"
#include "CalibrationRun.h"
#include "Analysis.h"
#include "DataMonitor.h"
#include "ScanConfig.h"
#include <ConfigFactory.h>
#include "utils.h"

CalibrationRun::CalibrationRun(DAQmanager *D)
    : RunManager(D)
{

}

CalibrationRun::~CalibrationRun()
{
}

void CalibrationRun::start_of_run()
{
    ALIBAVA_TRACE_CLASS("");
    ConfigFactory &cfg = ConfigFactory::instance();
    Analysis *ana = dynamic_cast<Analysis *>(cfg.get("analysis"));
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg.get("calibration"));
    ana->update_pedestals(false);

    DataMonitor *mon = get_daq_manager()->get_monitor();
    if (mon)
        mon->create_signal_profile("injected pulse",
                                   cal->npts(),
                                   cal->from(),
                                   cal->to() );
    get_daq_manager()->set_expected( cal->npts()*cal->nevts() );
    get_daq_manager()->set_received(0);
}

int CalibrationRun::loop( std::ostream &oheader)
{
    ALIBAVA_TRACE_CLASS("");
    unsigned int rc, i, step;
    ConfigFactory &cfg = ConfigFactory::instance();
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg.get("calibration"));
    step = cal->step();
    oheader << cal->npts() << "; "
            << cal->from() << "; "
            << cal->to()  << "; "
            << cal->step();
    if ( ! cal->scan_charge() )
        oheader << "; delay_scan";

    rc = 0;
    for (i=0; i<cal->npts();i++)
    {
        if (get_daq_manager()->stop())
            break;

        if ( (rc=acquire(cal->from()+ i *step)) )
            return rc;

        if ( (rc=read_data()) )
            return rc;

    }
    return 0;
}

int CalibrationRun::acquire(unsigned int val)
{
    ALIBAVA_TRACE_CLASS("");
    int rc = 0;
    ConfigFactory &cfg = ConfigFactory::instance();
    CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg.get("calibration"));
    if ( cal->scan_charge() )
        rc = get_daq_manager()->get_board()->calibration(val, cal->delay(), cal->nevts());
    else
        rc = get_daq_manager()->get_board()->calibration(cal->charge(), val, cal->nevts());

    return rc;
}
