#include <iostream>
#include "ConfigFile.h"

ConfigFile::ConfigFile(std::string g, KeyFile *f)
    : keyfile(f), group(g), emitter(0)
{
    emitter = emitter_new();
    g_object_ref(G_OBJECT(emitter));
}


ConfigFile::ConfigFile(const ConfigFile &F)
{
    cpy(F);
}
ConfigFile &ConfigFile::operator=(const ConfigFile &F)
{
    if (this!=&F)
        cpy(F);

    return *this;
}


ConfigFile::~ConfigFile()
{
    g_object_unref(G_OBJECT(emitter));
}

void ConfigFile::cpy(const ConfigFile &F)
{
    if (!emitter)
    {
        emitter = emitter_new();
        g_object_ref(G_OBJECT(emitter));
    }
    else
    {
//        g_object_unref(emitter);
//        emitter = F.emitter;
//        g_object_ref(G_OBJECT(emitter));
    }
    group = F.group;
    keyfile = F.keyfile;
}

void ConfigFile::set_file(KeyFile *f)
{
    keyfile = f;
    load_data();
}

bool ConfigFile::get_bool(const char *key, bool default_value)
{
    bool value=default_value;
    try
    {
        value = keyfile->get_as_bool(group, key);
    }
    catch (KeyFile::Exception &e)
    {
        std::cout << e.what() << std::endl;
    }
    return value;
}

void ConfigFile::set_bool(const char *key, bool value)
{
    keyfile->set_as_bool(group, key, value);
    g_signal_emit_by_name(emitter, "parameter-changed",  group.c_str(), key, NULL);
}

int ConfigFile::get_int(const char *key, int default_value)
{
    int value=default_value;
    try
    {
        value = keyfile->get_as_int(group, key);
    }
    catch (KeyFile::Exception &e)
    {
        std::cout << e.what() << std::endl;
    }
    return value;
}

void ConfigFile::set_int(const char *key, int value)
{
    keyfile->set_as_int(group, key, value);
    g_signal_emit_by_name(emitter, "parameter-changed",  group.c_str(), key, NULL);
}

double ConfigFile::get_double(const char *key, double default_value)
{
    double value=default_value;
    try
    {
        value = keyfile->get_as_double(group, key);
    }
    catch (KeyFile::Exception &e)
    {
        std::cout << e.what() << std::endl;
    }

    return value;
}

void ConfigFile::set_double(const char *key, double value)
{
    keyfile->set_as_double(group, key, value);
    g_signal_emit_by_name(emitter, "parameter-changed",  group.c_str(), key, NULL);
}

bool ConfigFile::exists() const
{
    return keyfile->has_group(group);
}

double *ConfigFile::get_double_list(const char *key, int &length)
{
    double *rc=0;
    length=0;
    try
    {
        int i;
        std::vector<double> v;
        keyfile->get_as_double_list(group, key, v);
        length = v.size();
        rc = new double[length];

        std::vector<double>::iterator ip;
        for (i=0, ip=v.begin(); ip!=v.end(); ++ip, i++)
        {
            rc[i] = *ip;
        }
    }
    catch (KeyFile::Exception &e)
    {
        std::cout << e.what() << std::endl;
    }
    return rc;
}
void ConfigFile::set_double_list(const char *key, double *v, int length)
{
    std::vector<double> vc;
    for (int i=0; i<length; i++)
        vc.push_back(v[i]);

    keyfile->set_as_double_list(group, key, vc);
    g_signal_emit_by_name(emitter, "parameter-changed",  group.c_str(), key, NULL);
}

std::string ConfigFile::get_string(const char *key)
{
    std::string rc;
    try
    {
        rc = keyfile->value(group, key);
    }
    catch (KeyFile::Exception &e)
    {
        std::cout << e.what() << std::endl;
    }
    return rc;
}

void ConfigFile::set_string(const char *key, const std::string &value)
{
    keyfile->add_key(group, key, value);
    g_signal_emit_by_name(emitter, "parameter-changed",  group.c_str(), key, NULL);
}

std::ostream &operator<<(std::ostream &os, const ConfigFile &C)
{
    const KeyFile *kf = C.key_file();
    if (kf)
        os << *kf;

    return os;
}

int ConfigFile::connect(GCallback func, gpointer data)
{
    return g_signal_connect(emitter, "parameter-changed", func, data);
}
