#include "DAQConfig.h"


DAQConfig::DAQConfig() :
ConfigFile("DAQ"), _chan(60), _nevt_per_sample(10), _nevts(10000),
_delay(60)
{
}
DAQConfig::DAQConfig(const DAQConfig &c)
{
    cpy(c);
}

DAQConfig::~DAQConfig()
{
}

void DAQConfig::cpy(const DAQConfig &c)
{
    *dynamic_cast<ConfigFile *>(this) = *dynamic_cast<const ConfigFile *>(&c);
    chan(c._chan);
    sample_size( c._nevt_per_sample );
    nevts( c._nevts );
    delay( c._delay );
}

DAQConfig &DAQConfig::operator=(const DAQConfig &b)
{
    if ( this!=&b)
        cpy(b);

    return *this;
}

int DAQConfig::chan() const
{
    return _chan;
}
void DAQConfig::chan(int x)
{
    _chan = x;
    set_int("monitor_channel", x);
}

int DAQConfig::sample_size() const
{
    return _nevt_per_sample;
}
void DAQConfig::sample_size(int x)
{
    _nevt_per_sample = x;
    set_int("sample_size", x);
}
int DAQConfig::nevts() const
{
    return _nevts;
}

void DAQConfig::nevts(int x)
{
    _nevts = x;
    set_int("nevts", x);
}
int DAQConfig::delay() const
{
    return _delay;
}
void DAQConfig::delay(int x)
{
    _delay = x;
    set_int("delay", x);
}


void DAQConfig::load_data()
{
    if (exists())
    {
        chan(get_int("monitor_channel") );
        sample_size(get_int("sample_size") );
        nevts(get_int("nevts"));
        delay(get_int("delay"));
    }
}

void DAQConfig::save_data()
{
    set_int("monitor_channel", chan());
    set_int("sample_size", sample_size());
    set_int("nevts", nevts());
    set_int("delay", delay());
}
