/*
 * DataFormat.cc
 *
 *  Created on: May 27, 2010
 *      Author: lacasta
 */
#include <cstring>
#include <iostream>
#include "DataFormat.h"

std::allocator<char> EventData::data_allocator;

EventData::EventData()
    : _value(0.0), _nchip(0), _firmware(0), _mask(0), psize(0)
{
    memset((void *)&block, 0, sizeof(block));
}


class OldFirmware : public EventData
{
    public:
        OldFirmware()
        {
            set_nchip(2);
        }
        virtual ~OldFirmware() {};
        int size_calib() { return 512; }
        int size_data() { return 518; }
        int read_data(USBport &);
};
int OldFirmware::read_data(USBport &port)
{
    int rc, out, i;
    int sz = size_calib()/2;

    for (i=0; i<16; i++)
        block.chip[0].header[i] = block.chip[1].header[i] = 0;

    rc = port.read( block.chip[0].signal, sz );
    if (rc < sz)
        return rc;

    out = port.read( block.chip[1].signal, sz );
    if ( out < 0)
        return out;

    return rc+out;
}


class NewFirmware : public EventData
{
    public:
        NewFirmware(int nchip)
        {
            set_nchip(nchip);
        }
        virtual ~NewFirmware() {}
        int size_calib() { return nchip()*288; }
        int size_data() { return 6 + nchip()*288; }
        int read_data(USBport &);
};

int NewFirmware::read_data(USBport &port)
{
    int rc, out=0;
    int sz = size_calib();
    rc = port.read( &block.chip[0], sz );
    return rc;
}


EventData::~EventData()
{
}

void EventData::set_nchip(int nc)
{
    _nchip = nc;
}




EventData *EventData::factory(int nchip, int fw, int mask)
{
    EventData *out = 0;
    if (fw <= 0)
        out = new OldFirmware();
    else
    {
        out = new NewFirmware(nchip);
    }
    out->_firmware = fw;
    out->_mask = mask;
    return out;
}
