#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstdlib>
#include <unistd.h>
#include <csignal>
#include <cerrno>
#include <cstring>
#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#ifdef __APPLE__
# include <mach/task.h>
# include <mach/mach_init.h>
#endif

#include "Progress.h"

static char sbuf[4096];
int Progress::page_k_shift=0;

//static void timeAlarm(int)
//{
//    signal(SIGALRM, timeAlarm);
//}

typedef std::pair<double, std::string> timeString;

static inline std::pair<double, std::string> time_units(double t)
        {
    timeString rc(t, "s");
    if (rc.first>86400.)
    {
        rc.first /= 86400.;
        rc.second = "d";
    }
    else if (rc.first>3600.)
    {
        rc.first /= 3600.;
        rc.second = "h";
    }
    else if (rc.first>60.)
    {
        rc.first/=60.;
        rc.second = "m";
    }
    else if (rc.first>1)
    {
        rc.second = "s";
    }
    else if (rc.first>1.e-3)
    {
        rc.first /= 1.e-3;
        rc.second = "ms";
    }
    else if (rc.first>1.e-6)
    {
        rc.first /= 1.e-6;
        rc.second = "us";
    }
    else
    {
        rc.first /= 1.e-9;
        rc.second = "ns";
    }
    return rc;
        }

std::string Progress::print_time(double t)
{
    std::ostringstream ostr;
    timeString units = time_units(t);
    ostr.setf(std::ios::fixed);
    ostr << std::setprecision(2) << units.first << " " << units.second;
    return ostr.str();
}

Progress::Progress(double a, double b, int opt) :
           flg(opt), usec(500000), thread(0), ss(true), last_value(a), last_update(0.), proc_file("")
{
    reset(a, b);

#ifndef G_OS_WIN32
    if (page_k_shift==0)
    {
        int j;
        for (j = getpagesize(); j > 1024; j >>= 1)
            page_k_shift++;
    }
#endif
}

Progress::~Progress()
{

}

void Progress::reset(double a, double b)
{
    value=0;
    vmin=a;
    vmax=b;
    if (vmin>vmax)
    {
        double tmp = vmin;
        vmin=vmax;
        vmax=tmp;
    }
}

void Progress::start()
{
    GError *error;
    ss = true;
    if (flg & timing)
        timer_start();

    mm0=mem();

    thread = g_thread_create( _loop, this, true, &error);
    // TODO: handle error
}

void Progress::stop()
{
    if (!thread)
        return;

    ss = false;
    if (flg & timing)
        timer_stop();

    while ( !ss)
        usleep( 1000);

    g_thread_join(thread);
    thread=0;
    std::cout << std::endl;
}

void Progress::show()
{
    if (flg==show_nothing)
        return;

    int width=-1;
    const char *sc = getenv("COLUMNS");
    if (sc)
    {
        std::istringstream istr(sc);
        istr >> width;
        if (!istr)
            width=80;
    }
    else
        width=80;

    double now = time();
    timeString tunits = time_units(now);
    std::string time_str = print_time(now);
    std::ostringstream out;
    out.setf(std::ios::fixed);
    out << '\r';
    if (flg & show_average)
    {
        timeString ts = time_units(value>0. ? now/(value) : 0.);
        out << std::setw(5) << std::setprecision(2) << ts.first << " " << ts.second <<"/upd ";
        last_value = value;
        last_update = now;
    }
    if ( flg & show_rate)
    {
        double rate = value/now;
        out << std::setw(6) << std::setprecision(1) << rate << " Hz ";
    }
    if (flg & show_time)
    {
        out << "Time: " << time_str << " ";
    }
    if (flg & show_left)
    {
        double tf = Percent();
        if (tf!=0.)
            tf = (1.-tf)*time()/tf;

        timeString ts = time_units(tf);
        out << "Left: " << std::setw(5) << std::setprecision(1) << ts.first << ts.second <<" ";
    }
    if (flg & show_memory)
    {
        out << "Mem.: " << std::setw(5) << std::setprecision(1) << (mem())/1024. << " Mb ";
    }
    int ncol, icol, i;
    if (flg & show_status)
    {
        ncol = width-15-out.str().size();
        icol = int(ncol*(value-vmin)/(vmax-vmin));
        out << " [";
        for (i=0; i<icol; i++)
            out << '#';
        for (; i<ncol; i++)
            out << ' ';
        out << "] ";
        if ( !(flg&show_value))
        {
            out << std::setw(5) << std::setprecision(1) << 100.*(value-vmin)/(vmax-vmin) << " % ";
        }
        else
        {
            out << std::setw(8) << std::setprecision(1) << value;
        }
    }

    std::cout << out.str() << std::flush;
}

void *Progress::_loop(void *ptr)
{
    Progress *prg = (Progress *)ptr;

    //    struct itimerval tval;
    //    signal (SIGALRM, timeAlarm);
    //    getitimer (ITIMER_REAL, &tval);
    //    tval.it_value.tv_sec = 0;
    //    tval.it_value.tv_usec = prg->usec;
    //    tval.it_interval = tval.it_value;
    //    setitimer (ITIMER_REAL, &tval, NULL);
    while (prg->ss)
    {
        usleep(prg->usec);
        prg->show();
    }
    prg->proc_file.clear();
    prg->ss=true;
    return 0;
}

int Progress::read_file()
{
#ifdef G_PLATFORM_WIN32
    return -1;
#else
    std::ostringstream ostr;
    ostr << "/proc/" << getpid() << "/statm" << std::ends;
    proc_file = ostr.str();

    int fd = open(proc_file.c_str(), O_RDONLY);
    if (fd < 0 && !proc_file.empty())
    {
        std::cout << "\n\nCould not open " << proc_file << std::endl << std::endl;
        return -1;
    }
    int iok, r = 0;
    *sbuf=0;
    do
    {
        iok = read(fd, sbuf, 4096);
        if (iok>0)
            r+=iok;
    } while (iok<0 && errno==EINTR);
    close(fd);
    return r;
#endif
}

float Progress::mem()
{
#ifdef __linux__
    int statlen;
    statlen = read_file();
    if (statlen>0)
    {
        unsigned long size;
        //       istringstream istr(sbuf,sizeof(sbuf));
        //       istr >> size;
        sscanf(sbuf, "%ld", &size);
        size <<= page_k_shift;
        return ((float)size);
    }
#endif

#ifdef __APPLE__
    // Inspired by:
    // http://miknight.blogspot.com/2005/11/resident-set-size-in-mac-os-x.html
    bool resident=false;
    struct task_basic_info t_info;
    mach_msg_type_number_t t_info_count = TASK_BASIC_INFO_COUNT;
    task_info(current_task(), TASK_BASIC_INFO, (task_info_t)&t_info, &t_info_count);
    size_t size = (resident ? t_info.resident_size : t_info.virtual_size);
    return ((float)size);
#endif

    return 0.;
}

