#include <iostream>
#include <sstream>
#include <cmath>
#include "ScanConfig.h"

void ScanConfig::cpy(const ScanConfig &c)
{
    set_group(c.get_group() );
    _from = c._from;
    _to = c._to;
    _npts = c._npts;
    _nevts= c._nevts;
    _min_step = c._min_step;
    adjust();
}

void ScanConfig::adjust()
{
    unsigned int tmp;
    if (_npts == 0)
        _npts = 1;

    if (_from > _to)
    {
        tmp = _from;
        _from = _to;
        _to = tmp;
    }
    double step = double(_to - _from)/double(_npts);
    
    // We stick to multiples of the minimum step
    double nstp = rint(step/_min_step);
    if (nstp==0)
        nstp = 1;
    
    step = nstp*_min_step;
    
    // now adjust the beginning of the range
    _from = int(rint(double(_from)/_min_step)*_min_step);
    _to   = int(_from + step*_npts);
    
}

void ScanConfig::load_data()
{
    if (exists())
    {
        from(get_int("from") );
        to(get_int("to") );
        npts(get_int("npts") );
        nevts(get_int("nevts") );
    }
}

void ScanConfig::save_data()
{
    set_int("from", from());
    set_int("to", to());
    set_int("npts", npts());
    set_int("nevts", nevts());
}


CalibrationConfig::CalibrationConfig(const CalibrationConfig &c)
{
    _delay = c._delay;
    _charge = c._charge;
    _scan_charge = c._scan_charge;
    cpy(c);
}
CalibrationConfig &CalibrationConfig::operator=(const CalibrationConfig &c)
{
    if (&c != this)
    {
        _delay = c._delay;
        _charge = c._charge;
        _scan_charge = c._scan_charge;
        cpy(c);
    }
    return *this;
}

void CalibrationConfig::load_data()
{
    if (exists())
    {
        ScanConfig::load_data();
        int dd = get_int("delay", 127);
        delay(dd);

        dd = get_int("charge", 100000);
        charge(dd);
    }
}

void CalibrationConfig::save_data()
{
    ScanConfig::save_data();
    set_int("delay", delay());
    set_int("charge", charge());
}
